﻿' The MIT License (MIT)
' 
' COPYRIGHT 2013. BLOOMBERG FINANCE L.P.
' 
' Permission is hereby granted, free of charge, to any person obtaining a copy
' of this software and associated documentation files (the "Software"), to deal
' in the Software without restriction, including without limitation the rights
' to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
' copies of the Software, and to permit persons to whom the Software is
' furnished to do so, subject to the following conditions:
' 
' The above copyright notice and this permission notice shall be included in
' all copies or substantial portions of the Software.
' 
' THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
' IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
' FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
' AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
' LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
' OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
' THE SOFTWARE.

Imports System.IO
Imports System.Reflection
Imports System.Threading
Imports System.Xml.XPath

Namespace PluginHost.Hosting
	Public Class PluginContainer
		Inherits MarshalByRefObject
		Private _args() As String

		Public Sub Launch(ByVal args() As String)
			_args = args

			Dim thread = New Thread(AddressOf Run)
			thread.TrySetApartmentState(ApartmentState.STA)
			thread.Start()
		End Sub

		Private Sub Run()
            Dim metadata = New XPathDocument(Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Metadata.xml")).CreateNavigator.SelectSingleNode("/Plugin")

			Debug.Assert(metadata IsNot Nothing)

			Dim mainAssembly = CStr(metadata.Evaluate("string(MainAssembly)"))
			Dim mainClass = CStr(metadata.Evaluate("string(MainClass)"))
			Dim title = CStr(metadata.Evaluate("string(Title)"))

			Debug.Assert((Not String.IsNullOrEmpty(mainAssembly)))
			Debug.Assert((Not String.IsNullOrEmpty(mainClass)))
			Debug.Assert((Not String.IsNullOrEmpty(title)))

			Dim rootElement = (CType(System.Delegate.CreateDelegate(GetType(Func(Of String(), UIElement)), System.Reflection.Assembly.Load(mainAssembly).GetType(mainClass), "CreateRootElement"), Func(Of String() , UIElement)))(_args)

			Dim window = New Window With {.SizeToContent = SizeToContent.WidthAndHeight, .Title = title, .Content = rootElement}

            CType(New Application, Application).Run(window)
			AppDomain.Unload(AppDomain.CurrentDomain)
		End Sub
	End Class
End Namespace
