﻿' The MIT License (MIT)
' 
' COPYRIGHT 2013. BLOOMBERG FINANCE L.P.
' 
' Permission is hereby granted, free of charge, to any person obtaining a copy
' of this software and associated documentation files (the "Software"), to deal
' in the Software without restriction, including without limitation the rights
' to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
' copies of the Software, and to permit persons to whom the Software is
' furnished to do so, subject to the following conditions:
' 
' The above copyright notice and this permission notice shall be included in
' all copies or substantial portions of the Software.
' 
' THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
' IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
' FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
' AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
' LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
' OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
' THE SOFTWARE.

Imports System.IO
Imports System.Reflection

Namespace PluginHost.Hosting
	Public Class PluginController
		Inherits MarshalByRefObject
		Private ReadOnly _appsRoot As String

		Public Sub New()
			_appsRoot = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Plugins")
		End Sub

		Public Sub Launch(ByVal commandLine As String)
			Dim tokens = commandLine.Split( {" "c}, StringSplitOptions.RemoveEmptyEntries)

			Dim mnemonic = tokens(0)
			If mnemonic.Length > 4 Then
				mnemonic = mnemonic.Substring(0, 4)
			End If

			Dim args = New String(tokens.Length - 2){}
			Array.Copy(tokens, 1, args, 0, args.Length)

			Dim appPath = Path.Combine(_appsRoot, mnemonic)

			Dim setup = New AppDomainSetup With {.ApplicationBase = appPath}
            Dim config = New RuntimeConfig

			Dim pluginContType = GetType(PluginContainer)
            Dim assemblyName = pluginContType.Assembly.GetName

			config.AddCodeBaseFor(assemblyName)

            For Each name In Directory.GetFiles("PAC", "*.dll").Select(AddressOf assemblyName.GetAssemblyName)
                config.AddCodeBaseFor(name)
            Next name
            setup.SetConfigurationBytes(config.GetBytes)

			Dim domain = AppDomain.CreateDomain(mnemonic, Nothing, setup)

			Dim host = CType(domain.CreateInstanceAndUnwrap(pluginContType.Assembly.FullName, pluginContType.FullName), PluginContainer)

			host.Launch(args)
		End Sub

		Public Overrides Function InitializeLifetimeService() As Object
			Return Nothing
		End Function
	End Class
End Namespace