﻿' The MIT License (MIT)
' 
' COPYRIGHT 2013. BLOOMBERG FINANCE L.P.
' 
' Permission is hereby granted, free of charge, to any person obtaining a copy
' of this software and associated documentation files (the "Software"), to deal
' in the Software without restriction, including without limitation the rights
' to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
' copies of the Software, and to permit persons to whom the Software is
' furnished to do so, subject to the following conditions:
' 
' The above copyright notice and this permission notice shall be included in
' all copies or substantial portions of the Software.
' 
' THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
' IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
' FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
' AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
' LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
' OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
' THE SOFTWARE.

Imports System.IO
Imports System.Reflection
Imports System.Text
Imports System.Xml

Namespace PluginHost.Hosting
	Friend Class RuntimeConfig
		Private Const BindingsNamespace As String = "urn:schemas-microsoft-com:asm.v1"
		Private Shared ReadOnly _encoding As Encoding = Encoding.UTF8

		Private ReadOnly _document As XmlDocument
		Private ReadOnly _binding As XmlElement

		Public Sub New()
            _document = New XmlDocument
			_document.AppendChild(_document.CreateXmlDeclaration("1.0", _encoding.WebName, Nothing))

			_binding = _document.CreateElement("assemblyBinding", BindingsNamespace)

			_document.AppendChild(_document.CreateElement("configuration")).AppendChild(_document.CreateElement("runtime")).AppendChild(_binding)
		End Sub

		Public Sub AddCodeBaseFor(ByVal name As AssemblyName)
			Dim assemblyElement = _document.CreateElement("dependentAssembly", BindingsNamespace)

			Dim identityElement = _document.CreateElement("assemblyIdentity", BindingsNamespace)

			identityElement.SetAttribute("name", name.Name)
			identityElement.SetAttribute("publicKeyToken", FormatPublicKeyToken(name))
			identityElement.SetAttribute("culture", FormatCulture(name))

			Dim codeBaseElement = _document.CreateElement("codeBase", BindingsNamespace)

            codeBaseElement.SetAttribute("version", name.Version.ToString)
			codeBaseElement.SetAttribute("href", name.CodeBase)

			assemblyElement.AppendChild(identityElement)
			assemblyElement.AppendChild(codeBaseElement)

			_binding.AppendChild(assemblyElement)
		End Sub

		Public Function GetBytes() As Byte()
            Using stream = New MemoryStream
                Using writer = New StreamWriter(stream, _encoding)
                    _document.Save(writer)
                    Return stream.ToArray
                End Using
            End Using
		End Function

		Private Shared Function FormatCulture(ByVal assemblyName As AssemblyName) As String
			Dim culture = assemblyName.CultureInfo.Name
			Return If(String.IsNullOrEmpty(culture), "neutral", culture)
		End Function

		Private Shared Function FormatPublicKeyToken(ByVal assemblyName As AssemblyName) As String
            Dim bytes = assemblyName.GetPublicKeyToken

			If bytes Is Nothing OrElse bytes.Length = 0 Then
				Return "null"
			End If
			Const hexDigits As String = "0123456789abcdef"
			Dim text = New StringBuilder(bytes.Length*2)

			For Each value In bytes
				text.Append(hexDigits.Chars(value >> 4))
                text.Append(hexDigits.Chars(value And 15))
			Next value
            Return text.ToString
		End Function
	End Class
End Namespace