﻿' The MIT License (MIT)
' 
' COPYRIGHT 2013. BLOOMBERG FINANCE L.P.
' 
' Permission is hereby granted, free of charge, to any person obtaining a copy
' of this software and associated documentation files (the "Software"), to deal
' in the Software without restriction, including without limitation the rights
' to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
' copies of the Software, and to permit persons to whom the Software is
' furnished to do so, subject to the following conditions:
' 
' The above copyright notice and this permission notice shall be included in
' all copies or substantial portions of the Software.
' 
' THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
' IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
' FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
' AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
' LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
' OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
' THE SOFTWARE.

Imports System.Windows.Controls.DataVisualization.Charting

Namespace DapiSample
	''' <summary>
	''' Interaction logic for MainView.xaml
	''' </summary>
	Partial Public Class MainView
		Private ReadOnly _marketDataProvider As MarketDataProvider

		Public Sub New()
			InitializeComponent()
			_chart.Visibility = Visibility.Hidden

			' set series style
		   Dim st = New Style(GetType(LineDataPoint))
			st.Setters.Add(New Setter(TemplateProperty, Nothing))
			_lineSeries.DataPointStyle = st

            _marketDataProvider = New MarketDataProvider
			AddHandler _marketDataProvider.MarketDataEventHandler, AddressOf OnMarketDataHandler
			AddHandler _marketDataProvider.MarketDataErrorHandler, AddressOf OnMarketDataError

            AddHandler Me.Unloaded, Sub(sender, args) _marketDataProvider.Dispose()
		End Sub

		Public Sub New(ByVal args() As String)
			Me.New()
			_security.Text = String.Join(" ", args)

			If _security.Text = String.Empty Then
                Exit Sub
			End If
			RequestData()
		End Sub

		Private Sub ButtonClick(ByVal sender As Object, ByVal e As RoutedEventArgs)
			RequestData()
		End Sub

		Private Sub RequestData()
			_marketDataProvider.RequestReferenceData(_security.Text, New Date(Date.Now.Year, 1, 1), Date.Now, "DAILY")
		End Sub

		Private Sub OnMarketDataError(ByVal [error] As String)
            Dispatcher.BeginInvoke(New Action(Sub()
                                                  Try
                                                      MessageBox.Show([error], "Error", MessageBoxButton.OK, MessageBoxImage.Error)
                                                  Catch ex As Exception
                                                      Console.WriteLine(ex.ToString)
                                                  End Try
                                              End Sub))
		End Sub

		Private Sub OnMarketDataHandler(ByVal series As List(Of DataPoint))
            Dispatcher.BeginInvoke(New Action(Sub()
                                                  Try
                                                      _chart.Visibility = Visibility.Hidden
                                                      SetLinearAxis(series)
                                                      _chart.DataContext = series
                                                      _chart.Visibility = Visibility.Visible
                                                  Catch ex As Exception
                                                      Console.WriteLine(ex.ToString)
                                                  End Try
                                              End Sub))
		End Sub

		Private Sub SetLinearAxis(ByVal series As List(Of DataPoint))
			_linearAxis.Minimum = 0

			Dim max As Double = series.Max(Function(val) val.Value)
			Dim min As Double = series.Min(Function(val) val.Value)

			Dim range As Double = CInt(Fix(max - min))
			Dim intervals As Integer

			If range < 100 Then
				intervals = 5
			Else
				intervals = 8
			End If

			Dim interval As Double = range / intervals

			If interval < 1 Then
				interval =.5
			Else
				interval = CInt(Fix(interval))
			End If


			_linearAxis.Maximum = max
			_linearAxis.Minimum = min

			_linearAxis.Interval = interval

		End Sub
	End Class
End Namespace
