﻿' The MIT License (MIT)
' 
' COPYRIGHT 2013. BLOOMBERG FINANCE L.P.
' 
' Permission is hereby granted, free of charge, to any person obtaining a copy
' of this software and associated documentation files (the "Software"), to deal
' in the Software without restriction, including without limitation the rights
' to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
' copies of the Software, and to permit persons to whom the Software is
' furnished to do so, subject to the following conditions:
' 
' The above copyright notice and this permission notice shall be included in
' all copies or substantial portions of the Software.
' 
' THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
' IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
' FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
' AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
' LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
' OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
' THE SOFTWARE.

Imports Bloomberglp.Blpapi

Namespace DapiSample
	Public Class MarketDataProvider
		Implements IDisposable
		Private ReadOnly _session As Session
		Private ReadOnly _refDataService As Service

		Public Event MarketDataEventHandler As Action(Of List(Of DataPoint))
		Public Event MarketDataErrorHandler As Action(Of String)

		Public Sub New()
            Dim sessionOptions = New SessionOptions With {.ServerHost = "localhost", .ServerPort = 8194, .ClientMode = Bloomberglp.Blpapi.SessionOptions.ClientModeType.DAPI, .AutoRestartOnDisconnection = True, .ConnectTimeout = 60000}

            _session = New Session(sessionOptions, AddressOf ProcessEventCallBack)

            If _session.Start Then
                If _session.OpenService("//blp/refdata") Then
                    _refDataService = _session.GetService("//blp/refdata")
                Else
                    Trace.WriteLine("c Reference Data Service")
                End If
            Else
                Trace.WriteLine("Failed to start Bloomberg Api session")
            End If
		End Sub

		Private Sub ProcessEventCallBack(ByVal eventObject As [Event], ByVal session As Session)
			Try
                If eventObject.Type = [Event].EventType.RESPONSE Then
                    OnProcessReferenceDataResponse(eventObject)
                    Exit Sub
                End If

				For Each msg In eventObject
                    Trace.WriteLine(msg.MessageType.ToString)
				Next msg
			Catch ex As Exception
			   Trace.WriteLine(String.Format("Exception during processing Bloomberg Data API Services Event: {0}", ex), "ERROR")
			End Try
		End Sub

		Public Sub RequestReferenceData(ByVal security As String, ByVal start As Date, ByVal [end] As Date, ByVal periodicity As String)
			Dim request As Request = _refDataService.CreateRequest("HistoricalDataRequest")
			Dim securities As Element = request.GetElement("securities")

			securities.AppendValue(security)
			Dim fields As Element = request.GetElement("fields")
			fields.AppendValue("PX_LAST")

			request.Set("periodicityAdjustment", "ACTUAL")
			request.Set("periodicitySelection", periodicity)

			request.Set("startDate", String.Format("{0}{1:D2}{2:D2}", start.Year, start.Month, start.Day))
			request.Set("endDate", String.Format("{0}{1:D2}{2:D2}", [end].Year, [end].Month, [end].Day))

            Trace.WriteLine("Sending Request: " & Val(request))
			_session.SendRequest(request, Nothing)
		End Sub


        Private Sub OnProcessReferenceDataResponse(ByVal eventObject As [Event])
            Dim series = New List(Of DataPoint)
            For Each msg As Message In eventObject
                Trace.WriteLine(msg.AsElement)

                Dim element = msg.AsElement

                Dim sd = element.GetElement("securityData")
                Dim fd = sd.GetElement("fieldData")

                If fd.NumValues > 0 Then
                    For i As Integer = 0 To fd.NumValues - 1
                        Dim val = fd.GetValue(i)
                        Dim price = CDbl((CType(val, Element)).GetElement("PX_LAST").GetValue)
                        Dim dt = CType((CType(val, Element)).GetElement("date").GetValue, Datetime)

                        series.Add(New DataPoint(New Date(dt.Year, dt.Month, dt.DayOfMonth), price))

                    Next i
                    RaiseEvent MarketDataEventHandler(series)
                Else 'error
                    RaiseEvent MarketDataErrorHandler(sd.GetElement("securityError").GetElement("message").ToString & sd.GetElement("security").ToString)
                    Trace.WriteLine(sd.GetElement("securityError").GetElement("message").ToString & sd.GetElement("security").ToString)
                End If
            Next msg
        End Sub

		Public Sub Dispose() Implements IDisposable.Dispose
			_session.Stop()
            CType(_session, IDisposable).Dispose()
		End Sub
	End Class
End Namespace
